@@ -1,5 +1,5 @@
 /* Convert from file to memory representation.
-   Copyright (C) 1998, 1999, 2000, 2002, 2012 Red Hat, Inc.
+   Copyright (C) 1998, 1999, 2000, 2002, 2012, 2015 Red Hat, Inc.
    This file is part of elfutils.
    Written by Ulrich Drepper <drepper@redhat.com>, 1998.
 
@@ -43,10 +43,8 @@
 
 
 Elf_Data *
-elfw2(LIBELFBITS, xlatetom) (dest, src, encode)
-     Elf_Data *dest;
-     const Elf_Data *src;
-     unsigned int encode;
+elfw2(LIBELFBITS, xlatetom) (Elf_Data *dest, const Elf_Data *src,
+			     unsigned int encode)
 {
   /* First test whether the input data is really suitable for this
      type.  This means, whether there is an integer number of records.
@@ -62,7 +60,7 @@ elfw2(LIBELFBITS, xlatetom) (dest, src, encode)
   /* We shouldn't require integer number of records when processing
      notes.  Payload bytes follow the header immediately, it's not an
      array of records as is the case otherwise.  */
-  if (src->d_type != ELF_T_NHDR
+  if (src->d_type != ELF_T_NHDR && src->d_type != ELF_T_NHDR8
       && src->d_size % recsize != 0)
     {
       __libelf_seterrno (ELF_E_INVALID_DATA);
