@@ -25,13 +25,13 @@
 #include <dwarf.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <error.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
 
+#include "system.h"
 #include "../libdw/known-dwarf.h"
 
 // The Dwarf, Dwarf_CFIs and address bias of
